
package org.negozio;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;


@javax.jws.WebService(
                      serviceName = "OrdiniImplService",
                      portName = "OrdiniImplPort",
                      targetNamespace = "http://www.negozio.org",
                      wsdlLocation = "Ordini.wsdl",
                      endpointInterface = "org.negozio.Ordini")
                      
public class OrdiniImpl implements Ordini {

    /**
     * Modificare login e database con il nome del proprio account.
     */
    private static final String SERVER = "squalo";
    private static final String DATABASE = "login";
    private static final String LOGIN = "login";
    private static final String PASSWORD = "";
    
    
    /**
     * Paga gli ordini di un utente.
     */
    public int buyOrdini(String codFisc) { 
    	// Risorse necessarie per la query in database
        Connection con = null;
    	PreparedStatement pstmt = null;
    	ResultSet rs = null;
    	int totale = 0;
        try {
        	con = getConnection(OrdiniImpl.SERVER, OrdiniImpl.DATABASE, OrdiniImpl.LOGIN, OrdiniImpl.PASSWORD);
        	// Gestisco manualmente le transazioni:
        	con.setAutoCommit(false);
        	
        	// Calcolo il prezzo degli ordini. Per semplicita' consideto gli ordini tutti
        	// di prezzo 1. Mi basta contarli.
			pstmt = con.prepareStatement("SELECT count(*) FROM ordini WHERE codfiscale=? AND pagato='false'");
			pstmt.setString(1, codFisc);
			rs = pstmt.executeQuery();
			while(rs.next()){
				totale = rs.getInt(1);
			}
			pstmt.close();
			// Adesso conosco il totale che il cliente deve pagare. Invio la richiesta di pagamento alla banca.
			// Simulo con una sleep di 5 secondi la gestione del pagamento verso la banca.
			Thread.sleep(5000);
			
			
			// Pagamento effettuato.
			// Imposto come pagati tutti gli ordini del cliente.
			pstmt = con.prepareStatement("UPDATE ordini SET pagato='true' WHERE codfiscale=?");
			pstmt.setString(1, codFisc);
			pstmt.executeUpdate();
			pstmt.close();
			
			// Eseguo la transazione
			con.commit();
			
		} catch (Exception ex) {
			// C'e' stato un errore, annullo la transazione
			try { con.rollback(); } catch (SQLException e) {}
			throw new RuntimeException(ex);
		} finally {
			// Chiudo le risorse usate
			if(rs != null) try{ rs.close(); } catch (Exception e) {}
			if(pstmt != null) try{ pstmt.close(); } catch (Exception e) {}
			if(con != null) try{ con.close(); } catch (Exception e) {}
		}
		
		return totale;
    }
    
    
    /**
     * Aggiunge un ordine in database
     */
    public void addOrdine(Ordine ordine) { 
    	// Risorse necessarie per la query in database
        Connection con = null;
    	PreparedStatement pstmt = null;
    	ResultSet rs = null;
        try {
        	// Inserisco l'ordine in database
        	con = getConnection(OrdiniImpl.SERVER, OrdiniImpl.DATABASE, OrdiniImpl.LOGIN, OrdiniImpl.PASSWORD);
        	
			pstmt = con.prepareStatement("INSERT INTO ordini (codfiscale, idordine) VALUES (?, ?)");
			pstmt.setString(1, ordine.getCodfisc());
			pstmt.setString(2, ordine.getIdOrdine());
			pstmt.executeUpdate();
		} catch (Exception ex) {
			throw new RuntimeException(ex);
		} finally {
			// Chiudo le risorse usate
			if(rs != null) try{ rs.close(); } catch (Exception e) {}
			if(pstmt != null) try{ pstmt.close(); } catch (Exception e) {}
			if(con != null) try{ con.close(); } catch (Exception e) {}
		}
    }
    
    

    /**
     * Restituisce la lista degli ordini di un utente
     */
    public ListaOrdini getListaOrdini(Utente utente) { 
    	
        // Instanzio l'oggetto ListaOrdini e imposto l'attributo codfisc
        ListaOrdini ordini = new ListaOrdini();
        ordini.setCodfisc(utente.getCodfisc());
        
        // Risorse necessarie per la query in database
        Connection con = null;
    	PreparedStatement stmt = null;
    	ResultSet rs = null;
        try {
        	// Ricerco in database tutti gli ordini di un certo codice fiscale
			con = getConnection(OrdiniImpl.SERVER, OrdiniImpl.DATABASE, OrdiniImpl.LOGIN, OrdiniImpl.PASSWORD);
			stmt = con.prepareStatement("SELECT * FROM ordini WHERE codfiscale=?");
			stmt.setString(1, utente.getCodfisc());
			rs = stmt.executeQuery();
			
			// Scorro gli idOrdine e li aggiungo alla ListaOrdini
			while(rs.next()){
				Ordine o = new Ordine();
				o.setCodfisc(rs.getString("codfiscale"));
				o.setPagato(rs.getBoolean("pagato"));
				o.setIdOrdine(rs.getString("idOrdine"));
				ordini.getIdOrdine().add(o);
			}
		} catch (Exception ex) {
			throw new RuntimeException(ex);
		} finally {
			// Chiudo le risorse usate
			if(rs != null) try{ rs.close(); } catch (Exception e) {}
			if(stmt != null) try{ stmt.close(); } catch (Exception e) {}
			if(con != null) try{ con.close(); } catch (Exception e) {}
		}
		
		return ordini;
        
    }
    
    /**
     * Metodo per recuperare la connessione dal Database. 
     * 
     * @return
     * @throws ClassNotFoundException
     * @throws SQLException
     */
    public Connection getConnection(String server, String database, String login, String password) throws ClassNotFoundException, SQLException { 
    	// Carica la classe del driver JDBC
    	Class.forName("org.postgresql.Driver");

    	// Ottengo connessione al database
    	Connection connectionDB = DriverManager.getConnection("jdbc:postgresql://" + server + "/"+database, login,password);
    	if(connectionDB==null){
    	    System.err.println("Connection is null");
    	}
    	
    	// Ogni operazione viene immediatamente riportata sulla base di dati
    	connectionDB.setAutoCommit(true);
    	
    	return connectionDB;
    }

}
