
package org.negozio.client;

import java.net.URL;
import java.util.List;
import java.util.Random;

import javax.xml.namespace.QName;


import org.negozio.ListaOrdini;
import org.negozio.Ordine;
import org.negozio.Ordini;
import org.negozio.Utente;

import org.negozio.OrdiniImplService;

public final class OrdiniClient {

    private static final QName SERVICE_NAME = new QName("http://www.negozio.org", "OrdiniImplService");
    private static Ordini port = null ;
    private OrdiniClient() {
    }

    public static void main(String args[]) throws Exception {
        URL wsdlURL = new URL("file:WebContent/Ordini.wsdl");
        String codFisc = "AAABBB0001";
        OrdiniImplService ss = new OrdiniImplService(wsdlURL, SERVICE_NAME);
        port = ss.getOrdiniImplPort();  
        
        // Aggiungo 3 ordini.
        addOrdine(codFisc);
        addOrdine(codFisc);
        addOrdine(codFisc);
        Thread.sleep(1000);
        
        // Stampo la lista degli ordini
        printListaOrdini(codFisc);
        
        // Avvio in parallelo l'acquisto di altri 3 ordini
        new OrdiniClient().new buyerThread(codFisc).start();
        
        // Pago tutti gli ordini ancora non pagati
        buyOrdini(codFisc);
        
        // Stampo la lista degli ordini
        printListaOrdini(codFisc);
        
        System.exit(0);
    }

    /**
     * Richiede l'inserimento di un nuovo ordine per un certo codice fiscale.
     * @param codFisc
     */
    private static void addOrdine(String codFisc){
    	Random random = new Random();
    	int idOrdine = random.nextInt(5000);
    	System.out.println("Richiesta aggiunta ordine [CF:" + codFisc + "][idOrdine:" + idOrdine + "]");
        Ordine ordine = new Ordine();
        ordine.setCodfisc(codFisc);
        ordine.setIdOrdine(idOrdine + "");
        port.addOrdine(ordine);
    }
    
    /**
     * Stampa la lista degli ordini del codiceFiscale passato.
     * @param codFisc
     */
    
    private static void printListaOrdini(String codFisc){
    	System.out.println("Lista Ordini per l'utente [" + codFisc + "] ");
    	System.out.println("-----------------------------------");
    	System.out.println("\tidOrdine\tpagato");
        Utente utente = new Utente();
        utente.setCodfisc(codFisc);
        ListaOrdini listaOrdini = port.getListaOrdini(utente);
        List<Ordine> idOrdini = listaOrdini.getIdOrdine();
        int daPagare = 0;
        for(int i=0; i<idOrdini.size(); i++){
        	System.out.println("\t" + idOrdini.get(i).getIdOrdine() + "\t\t" + idOrdini.get(i).getPagato());
        	if(!idOrdini.get(i).getPagato()){
        		daPagare++;
        	}
        }
        System.out.println("Ordini da pagare: " + daPagare);
    }
    
    /**
     * Richiede il pagamento di tutti gli ordini di un codiceFiscale
     * @param codfisc
     */
    private static void buyOrdini(String codfisc){
    	System.out.println("Richiesto pagamento ordini....");
    	int pagati = port.buyOrdini(codfisc);
    	System.out.println("... pagamento ordini effettuato. Pagati " + pagati + " ordini.");
    }
    
    
    /**
     * Thread per l'esecuzione di acquisti in parallelo.
     */
    public class buyerThread extends Thread {
    	String codfiscale = null;
    	public buyerThread(String codfiscale){
    		this.codfiscale = codfiscale;
    	}
    	
    	public void run(){
    		try{ Thread.sleep(1000); } catch(InterruptedException e) {}
    		OrdiniClient.addOrdine(codfiscale);
    		OrdiniClient.addOrdine(codfiscale);
    		OrdiniClient.addOrdine(codfiscale);
    	}
    }
    
}